/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Torisugari.
 *
 * Contributor(s):
 *  Torisugari (Original author)
 *  Nickolay Ponomarev <asqueella@gmail.com>
 *  Malte Kraus <firefox@maltekraus.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*******  Dialog interface  ***************************************************/

const NS_ERROR_MODULE_NETWORK = 0x804b0000;

let gSilent; // whether to close on error
let gSyncService;

function loadScript(url, context) {
  if(url.indexOf(":/") == -1) url = "chrome://adblockfilters/content/" + url;
  if(!context && loadScript.caller)
    context = loadScript.caller.__parent__;
  if(!context)
    context = null;
  Cc["@mozilla.org/moz/jssubscript-loader;1"]
    .getService(Ci.mozIJSSubScriptLoader).loadSubScript(url, context);
}

function onLoad() {
  Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService)
    .addObserver(observer, "adblockfilters-sync-finished", false);
  let job = window.arguments ? window.arguments[0] : null;
  if(job) {
    setStatusText("init", null);
    job.notificationCallbacks = gNotificationCallbacks;
    setTimeout(function() { gSyncService = new SyncService(job); }, 0);
  } else {
    setStatusText("unexpected", -1);
    window.getAttention();
  }
}

var gNotificationCallbacks = {
  // nsIProgressEventSink
  onProgress: function(request, context, progress, progressMax) {
      var progress = document.getElementById("progress");
      progress.value = progress / progressMax * 100;
      progress.mode = "determined";
  },
  onStatus: function(request, context, status, statusArg) {},
  // nsISupports
  QueryInterface: function(aIID) {
    if(aIID.equals(Ci.nsISupports) || aIID.equals(Ci.nsIProgressEventSink))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE;
  }
};
var observer = {
  observe: function observe(job, topic, errCode) {
    job = job.wrappedJSObject;
    if(topic != "adblockfilters-sync-finished")
      return;
    Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService)
                                         .removeObserver(this, topic, false);
    flagedExit(parseInt(errCode));
  }
};

const netErrorStrings = {
  // ftp:
  "1": "unexpected",
  "2": "usercancel",
  "13": "refused",
  "14": "netTimeout",
  "16": "netOffline",
  "21": "ftplogin",
  "22": "ftpcwd",
  "23": "ftppasv",
  "24": "ftppwd",
  "25": "ftplist",
  "30": "unknown",
 // others listed in nsNetError.h

  // http:
  "401": "unauthorized",
  "403": "forbidden",
  "404": "notFound",
  "405": "methodNotAllowed",
  "412": "preconditionFailed",
  "500": "internalServerError"
};
const notNetErrorStrings = {
  //"-1": "nodataoruri",
  //"-2":"",
  "-3": "invalid-download"
};

function getErrorStr(aCode) {
  if(aCode == 0)
    return "done";

  LOG("error code: " + aCode);
  if(aCode > NS_ERROR_MODULE_NETWORK && aCode < NS_ERROR_MODULE_NETWORK + 0x10000) {
    aCode = (aCode % NS_ERROR_MODULE_NETWORK).toString();
    return netErrorStrings[aCode] || "unexpected";
  }
  aCode = aCode.toString();

  return netErrorStrings[aCode] || notNetErrorStrings[aCode] || "notneterror";
}

function getLocalizedString(aStr) {
  try {
    return document.getElementById("strbundle").getString(aStr);
  } catch(e) {}
  return aStr;
}

// aMsg is either a blank string or one of strings from neterrors.properties,
// aErrCode is either undefined or an error code.
// At least one of params must be defined.
function setStatusText(aMsg, aErrCode) {
  if(!document)
    return;
  var status;
  if(aMsg) {
    status = getLocalizedString(aMsg);
  } else {
    aMsg = getErrorStr(aErrCode);
    status = getLocalizedString(aMsg);
  }

  if(aMsg == "notneterror" || aMsg == "unexpected") {
    if(aErrCode > 0x80000000) {
      // aErrCode = (i for(i in Cr) if(Cr[i] == aErrCode)).next();
      for(let name in Cr) {
        if(Cr[name] == aErrCode) {
          aErrCode = name;
          break;
        }
      }
    }
    status += " (" + aErrCode + ")";
  }

  var item = document.getElementById("status");
  item.setAttribute("tooltiptext", item.value = status);
  var progress = document.getElementById("progress");
  progress.value = (aErrCode == 0) ? 100 : 0;
  progress.mode = "determined";
}

// updates status text and sets close-window timeout if
// aCode does not indicate that an error has occured
function flagedExit(aCode) {
  if(!document)
    return;
  setStatusText("", aCode);
  const DELAY = 2000;

  let(elem = document.documentElement) {
    elem.getButton("cancel").hidden = true;
    elem.getButton("accept").hidden = false;
  }

  function closeWindow() { window.close(); } // native functions can't be passed to setTimeout
  if(window.arguments[1]) {     // silent mode
    if(aCode == 0)              // suCcess - close immediately
      setTimeout(closeWindow, 0);
    else                        // failure - close after delay
      setTimeout(closeWindow, DELAY);
  } else {                      // normal mode
    if(aCode == 0) {            // suCcess - close after delay
      setTimeout(closeWindow, DELAY);
    } else {                    // failure - don't close
      window.getAttention();
      Cc["@mozilla.org/sound;1"].createInstance(Ci.nsISound).beep();
    }
  }
}

function LOG(aStr) {
  //dump(aStr);

  var consoleService = Cc["@mozilla.org/consoleservice;1"]
                         .getService(Ci.nsIConsoleService);
  consoleService.logStringMessage(aStr);
}